# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from collections import OrderedDict
from datetime import datetime
from inspect import currentframe, getouterframes
import json
import logging

import requests

from constants import DEFAULT_URL, TIMEOUT
from general import General
from model import Model
from actions import Actions
from groups import Groups
from customers import Customers
from segments import Segments
from integrations import Integrations


LOGGER = logging.getLogger(__name__)


class Client:
    token = None
    expire = None

    def __init__(self, username=None, password=None, timeout=TIMEOUT, url=DEFAULT_URL):
        self.general = General(self)
        self.model = Model(self)
        self.actions = Actions(self)
        self.groups = Groups(self)
        self.customers = Customers(self)
        self.segments = Segments(self)
        self.integrations = Integrations(self)
        self.timeout = timeout
        self._url = url

        if username and password:
            self.general.login(username, password)

    def _headers(self):
        headers = {
            'Accept': 'application/json',
            'Content-type': 'application/json'
        }

        if self.token:
            headers['Authorization-Token'] = self.token

        return headers

    def _refresh_token(self):
        if not self.expire or (self.expire - datetime.utcnow()).seconds >= 1200:
            self.general.login(self.general.username, self.general.password)
        return

    def get(self, url, payload=None, headers=None, check_token=True):
        if check_token:
            self._refresh_token()

        headers = headers if headers else self._headers()
        LOGGER.debug("GET request: url=%s, payload=%s, headers=%s", url, payload, headers)

        if payload:
            payload = OrderedDict(sorted(payload.items(), key=lambda t: t[0]))

        try:
            response = requests.get(url, params=payload, headers=headers, timeout=self.timeout)
        except requests.exceptions.Timeout as error:
            LOGGER.error("Timeout reached, error=%s", error)
            raise Exception('Timeout reached {}sec, url is {}'.format(self.timeout, url))

        LOGGER.debug("GET response: url=%s, response_data=%s", response.url, response.text)
        return self.dispatch_response(response)

    def post(self, url, payload=None, headers=None, check_token=True):
        if check_token:
            self._refresh_token()

        headers = headers if headers else self._headers()
        data = json.dumps(payload)
        LOGGER.debug("POST request: url=%s, data=%s, headers=%s", url, payload, headers)

        try:
            response = requests.post(url, data=data, headers=headers, timeout=self.timeout)
        except requests.exceptions.Timeout as error:
            LOGGER.error("Timeout reached, error=%s", error)
            raise Exception('Timeout reached {}sec, url is {}'.format(self.timeout, url))

        LOGGER.debug("POST response: url=%s, response_data=%s", url, response.text)
        return self.dispatch_response(response)

    @staticmethod
    def bad_request():
        raise Exception('Bad Request')

    @staticmethod
    def unauthorized():
        raise Exception('Unauthorized')

    @staticmethod
    def method_not_allowed():
        raise Exception('Method Not allowed')

    @staticmethod
    def internal_server_error():
        raise Exception('Internal Server error')

    def dispatch_response(self, response):
        if response.status_code == 200:
            return response
        elif response.status_code == 400:
            return self.bad_request()
        elif response.status_code == 401:
            return self.unauthorized()
        elif response.status_code == 405:
            return self.method_not_allowed()
        elif response.status_code == 500:
            return self.internal_server_error()
        else:
            return False

    def get_url(self):
        outer_frames = getouterframes(currentframe(), 2)[1]
        category_name = outer_frames[1].split('/')[-1].split('.')[0]
        method_name = outer_frames[3]
        if '_' in method_name:
            action_name_list = [part.upper() if part == 'id' else part.capitalize() for part in method_name.split('_')]
            action_name = ''.join(action_name_list)
        else:
            action_name = method_name
        return '%s/current/%s/%s' % (self._url, category_name, action_name)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


class Groups(object):
    client = None

    def __init__(self, client):
        self.client = client

    def get_target_group_name(self, target_group_id):
        """Returns the target group name associated with a particular target group ID."""
        if not target_group_id:
            raise Exception('No TargetGroupID provided')

        data = {
            'TargetGroupID': target_group_id
        }

        response = self.client.get(self.client.get_url(), data)
        return response.json()['TargetGroupName'] if response else False

    def get_target_group_id(self, target_group_name):
        """Returns the target group ID associated with a particular target group name."""
        if not target_group_name:
            raise Exception('No TargetGroupName provided')

        data = {
            'TargetGroupName': target_group_name
        }

        response = self.client.get(self.client.get_url(), data)
        return response.json()['TargetGroupID'] if response else False

    def get_target_groups_by_date(self, date):
        """Returns the list of target group IDs for which an acion was executed on a particular date."""
        if not date:
            raise Exception('No Date provided')

        data = {
            'Date': date
        }

        response = self.client.get(self.client.get_url(), data)
        return [item['TargetGroupID'] for item in response.json()] if response else False

    def get_target_group_details(self):
        """Returns an array of IDs, names and priorities for all defined target groups."""
        response = self.client.get(self.client.get_url())

        results = {}
        for item in response.json():
            results[item['TargetGroupID']] = {
                'name': item['TargetGroupName'],
                'priority': item['TargetGroupPriority']
            }

        return results



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import json
import unittest
from urlparse import parse_qs, urlparse

from optimove.client import Client
from optimove.constants import DEFAULT_URL
import responses

from constants import HEADERS
from helpers import login_callback, token_required


"""Callbacks"""


@token_required
def get_action_name_callback(request):
    params = parse_qs(urlparse(request.url).query)
    if params['ActionID'][0] == '104':
        resp_body = {'ActionName': '10% summer bonus'}
        return 200, HEADERS['json'], json.dumps(resp_body)
    else:
        return 404, HEADERS['text'], 'Not Found'


@token_required
def get_action_id_callback(request):
    params = parse_qs(urlparse(request.url).query)
    if params['ActionName'][0] == 'Free Shipping':
        resp_body = {'ActionID': 25}
        return 200, HEADERS['json'], json.dumps(resp_body)
    else:
        return 404, HEADERS['text'], 'Not Found'


@token_required
def get_all_actions_callback(request):
    resp_body = [
        {'ActionID': 1, 'ActionName': '10% Bonus'},
        {'ActionID': 2, 'ActionName': '15% Bonus'},
        {'ActionID': 3, 'ActionName': '20% Bonus'}
    ]
    return 200, HEADERS['json'], json.dumps(resp_body)


@token_required
def get_actions_by_target_group_callback(request):
    params = parse_qs(urlparse(request.url).query)

    if params['TargetGroupID'][0] == '7' and params['Date'][0] == '2015-10-23':
        resp_body = [
            {'RecipientGroupID': 1, 'ActionID': 23},
            {'RecipientGroupID': 2, 'ActionID': 27}
        ]
        return 200, HEADERS['json'], json.dumps(resp_body)

    else:
        return 404, HEADERS['text'], 'Not Found'


@token_required
def get_promo_codes_callback(request):
    params = parse_qs(urlparse(request.url).query)

    if params['Date'][0] == '2015-02-20':
        resp_body = [
            {'TargetGroupID': 9, 'RecipientGroupID': 1, 'ActionID': 24, 'PromoCode': 'HEP-FEB'},
            {'TargetGroupID': 9, 'RecipientGroupID': 2, 'ActionID': 25, 'PromoCode': 'HEP-FCC'},
            {'TargetGroupID': 13, 'RecipientGroupID': 1, 'ActionID': 65, 'PromoCode': 'GDG-FAL'}
        ]
        return 200, HEADERS['json'], json.dumps(resp_body)

    else:
        return 404, HEADERS['text'], 'Not Found'


@token_required
def get_promo_codes_by_campaign_callback(request):
    params = parse_qs(urlparse(request.url).query)

    if params['CampaignID'][0] == '721':
        resp_body = [
            {'RecipientGroupID': 1, 'ActionID': 24, 'PromoCode': 'HEP-75-FEB'},
            {'RecipientGroupID': 2, 'ActionID': 65, 'PromoCode': 'GDG-50-FRT'}
        ]
        return 200, HEADERS['json'], json.dumps(resp_body)

    else:
        return 404, HEADERS['text'], 'Not Found'


@token_required
def get_promo_codes_by_target_group_callback(request):
    params = parse_qs(urlparse(request.url).query)

    if params['TargetGroupID'][0] == '41' and params['Date'][0] == '2015-09-23':
        resp_body = [
            {'RecipientGroupID': 1, 'ActionID': 24, 'PromoCode': 'HEP-FEB'},
            {'RecipientGroupID': 2, 'ActionID': 25, 'PromoCode': 'HEP-FCC'},
            {'RecipientGroupID': 1, 'ActionID': 65, 'PromoCode': 'GDG-FAL'}
        ]
        return 200, HEADERS['json'], json.dumps(resp_body)

    else:
        return 404, HEADERS['text'], 'Not Found'


@token_required
def get_action_details_by_target_group_callback(request):
    params = parse_qs(urlparse(request.url).query)

    if params['TargetGroupID'][0] == '9' and params['Date'][0] == '2015-11-11':
        resp_body = [
            {'RecipientGroupID': 1, 'ActionID': 65, 'Duration': 7, 'LeadTime': 0, 'ChannelID': 1},
            {'RecipientGroupID': 1, 'ActionID': 78, 'Duration': 7, 'LeadTime': 0, 'ChannelID': 3}
        ]
        return 200, HEADERS['json'], json.dumps(resp_body)

    else:
        return 404, HEADERS['text'], 'Not Found'


@token_required
def get_executed_campaign_details_callback(request):
    params = parse_qs(urlparse(request.url).query)

    if params['Date'][0] == '2015-06-19':
        resp_body = [
            {'CampaignID': 221, 'TargetGroupID': 15, 'CampaignType': 'Test/Control', 'Duration': 7,
             'LeadTime': 3, 'Notes': '', 'IsMultiChannel': 'false', 'IsRecurrence': 'false',
             'Status': 'Successful', 'Error': ''},
            {'CampaignID': 81, 'TargetGroupID': 40, 'CampaignType': 'Test/Control', 'Duration': 10,
             'LeadTime': 0, 'Notes': '', 'IsMultiChannel': 'true', 'IsRecurrence': 'true',
             'Status': 'Failed', 'Error': 'ESP unavailable'}
        ]
        return 200, HEADERS['json'], json.dumps(resp_body)

    else:
        return 404, HEADERS['text'], 'Not Found'


@token_required
def get_campaign_details_callback(request):
    params = parse_qs(urlparse(request.url).query)

    if params['CampaignID'][0] == '21':
        resp_body = {'TargetGroupID': 15, 'CampaignType': 'Test/Control', 'Duration': 7,
                     'LeadTime': 3, 'Notes': '', 'IsMultiChannel': 'false', 'IsRecurrence': 'false',
                     'Status': 'Successful', 'Error': ''}
        return 200, HEADERS['json'], json.dumps(resp_body)

    else:
        return 404, HEADERS['text'], 'Not Found'


@token_required
def get_execution_channels_callback(request):
    resp_body = [
        {'ChannelID': 1, 'ChannelName': 'Silverpop'},
        {'ChannelID': 2, 'ChannelName': 'ExactTarget'},
        {'ChannelID': 3, 'ChannelName': 'Lobby Banner'},
        {'ChannelID': 4, 'ChannelName': 'Call Center'}
    ]
    return 200, HEADERS['json'], json.dumps(resp_body)


@token_required
def get_executed_campaign_channel_details_callback(request):
    params = parse_qs(urlparse(request.url).query)

    if params['CampaignID'][0] == '9214' and params['ChannelID'][0] == '35':
        resp_body = [
            {'ListID': 12, 'SendID': 'YTGF3C', 'TemplateID': 520, 'ScheduledTime': '2015-12-30 08:00:00'},
            {'TemplateID': 27, 'ScheduledTime': '2016-08-23 16:00:00', 'SendID': ''},
            {'TemplateID': 1321, 'ScheduledTime': '2016-08-23 18:00:00', 'SendID': ''}
        ]
        return 200, HEADERS['json'], json.dumps(resp_body)

    else:
        return 404, HEADERS['text'], 'Not Found'


@token_required
def get_executed_campaigns_by_channel_callback(request):
    params = parse_qs(urlparse(request.url).query)

    if params['ChannelID'][0] == '35' and params['Date'][0] == '2016-02-20':
        resp_body = [
            {'CampaignID': 12}, {'CampaignID': 16}, {'CampaignID': 17}, {'CampaignID': 19}
        ]
        return 200, HEADERS['json'], json.dumps(resp_body)

    else:
        return 404, HEADERS['text'], 'Not Found'


"""Tests"""


class TestActions(unittest.TestCase):

    @responses.activate
    def test_get_action_name(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetActionName',
            callback=get_action_name_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.actions.get_action_name(104)
        self.assertEqual(data, '10% summer bonus')

    @responses.activate
    def test_get_action_name_with_empty_action_id(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetActionName',
            callback=get_action_name_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        self.assertRaises(Exception, client.actions.get_action_name, None)

    @responses.activate
    def test_get_action_id(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetActionID',
            callback=get_action_id_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.actions.get_action_id('Free Shipping')
        self.assertEqual(data, 25)

    @responses.activate
    def test_get_action_id_with_empty_action_name(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetActionID',
            callback=get_action_id_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        self.assertRaises(Exception, client.actions.get_action_id, None)

    @responses.activate
    def test_get_all_actions(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetAllActions',
            callback=get_all_actions_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.actions.get_all_actions()
        self.assertEqual(data, {
            1: '10% Bonus',
            2: '15% Bonus',
            3: '20% Bonus'
        })

    @responses.activate
    def test_get_actions_by_target_group(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetActionsByTargetGroup',
            callback=get_actions_by_target_group_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.actions.get_actions_by_target_group(7, '2015-10-23')
        self.assertEqual(data, {
            1: 23,
            2: 27
        })

    @responses.activate
    def test_get_actions_by_target_group_with_empty_date(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetActionsByTargetGroup',
            callback=get_actions_by_target_group_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        self.assertRaises(Exception, client.actions.get_actions_by_target_group, 7, None)

    @responses.activate
    def test_get_actions_by_target_group_with_wrong_date(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetActionsByTargetGroup',
            callback=get_actions_by_target_group_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.actions.get_actions_by_target_group(7, '3015-10-23')
        self.assertFalse(data)

    @responses.activate
    def test_get_promo_codes(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetPromoCodes',
            callback=get_promo_codes_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.actions.get_promo_codes('2015-02-20')
        self.assertEqual(data, [
            {'target_group_id': 9, 'recipient_group_id': 1, 'action_id': 24, 'promo_code': 'HEP-FEB'},
            {'target_group_id': 9, 'recipient_group_id': 2, 'action_id': 25, 'promo_code': 'HEP-FCC'},
            {'target_group_id': 13, 'recipient_group_id': 1, 'action_id': 65, 'promo_code': 'GDG-FAL'}
        ])

    @responses.activate
    def test_get_promo_codes_with_empty_date(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetPromoCodes',
            callback=get_promo_codes_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        self.assertRaises(Exception, client.actions.get_promo_codes, None)

    @responses.activate
    def test_get_promo_codes_with_wrong_date(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetPromoCodes',
            callback=get_promo_codes_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.actions.get_promo_codes('3015-02-20')
        self.assertFalse(data)

    @responses.activate
    def test_get_promo_codes_by_campaign(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetPromoCodesByCampaign',
            callback=get_promo_codes_by_campaign_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.actions.get_promo_codes_by_campaign(721)
        self.assertEqual(data, [
            {'campaign_id': 721, 'recipient_group_id': 1, 'action_id': 24, 'promo_code': 'HEP-75-FEB'},
            {'campaign_id': 721, 'recipient_group_id': 2, 'action_id': 65, 'promo_code': 'GDG-50-FRT'}
        ])

    @responses.activate
    def test_get_promo_codes_by_campaign_with_empty_campaign_id(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetPromoCodesByCampaign',
            callback=get_promo_codes_by_campaign_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        self.assertRaises(Exception, client.actions.get_promo_codes_by_campaign, None)

    @responses.activate
    def test_get_promo_codes_by_campaign_with_wrong_campaign_id(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetPromoCodesByCampaign',
            callback=get_promo_codes_by_campaign_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.actions.get_promo_codes_by_campaign(127)
        self.assertFalse(data)

    @responses.activate
    def test_get_promo_codes_by_target_group(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetPromoCodesByTargetGroup',
            callback=get_promo_codes_by_target_group_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.actions.get_promo_codes_by_target_group(41, '2015-09-23')
        self.assertEqual(data, [
            {'target_group_id': 41, 'recipient_group_id': 1, 'action_id': 24, 'promo_code': 'HEP-FEB'},
            {'target_group_id': 41, 'recipient_group_id': 2, 'action_id': 25, 'promo_code': 'HEP-FCC'},
            {'target_group_id': 41, 'recipient_group_id': 1, 'action_id': 65, 'promo_code': 'GDG-FAL'}
        ])

    @responses.activate
    def test_get_promo_codes_by_target_group_with_empty_date(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetPromoCodesByTargetGroup',
            callback=get_promo_codes_by_target_group_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        self.assertRaises(Exception, client.actions.get_promo_codes_by_target_group, 41, None)

    @responses.activate
    def test_get_promo_codes_by_target_group_with_wrong_date(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetPromoCodesByTargetGroup',
            callback=get_promo_codes_by_target_group_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.actions.get_promo_codes_by_target_group(41, '3015-09-23')
        self.assertFalse(data)

    @responses.activate
    def test_get_action_details_by_target_group(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetActionDetailsByTargetGroup',
            callback=get_action_details_by_target_group_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.actions.get_action_details_by_target_group(9, '2015-11-11')
        self.assertEqual(data, [
            {'target_group_id': 9, 'recipient_group_id': 1, 'action_id': 65,
             'duration': 7, 'lead_time': 0, 'channel_id': 1},
            {'target_group_id': 9, 'recipient_group_id': 1, 'action_id': 78,
             'duration': 7, 'lead_time': 0, 'channel_id': 3},
        ])

    @responses.activate
    def test_get_action_details_by_target_group_with_empty_date(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetActionDetailsByTargetGroup',
            callback=get_action_details_by_target_group_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        self.assertRaises(Exception, client.actions.get_action_details_by_target_group, 9, None)

    @responses.activate
    def test_get_action_details_by_target_group_with_wrong_date(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetActionDetailsByTargetGroup',
            callback=get_action_details_by_target_group_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.actions.get_action_details_by_target_group(9, '3015-11-11')
        self.assertFalse(data)

    @responses.activate
    def test_get_executed_campaign_details(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetExecutedCampaignDetails',
            callback=get_executed_campaign_details_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.actions.get_executed_campaign_details('2015-06-19')
        self.assertEqual(data, {
            221: {
                'target_group_id': 15,
                'campaign_type': 'Test/Control',
                'duration': 7,
                'lead_time': 3,
                'notes': '',
                'is_multi_channel': False,
                'is_recurrence': False,
                'status': 'Successful',
                'error': '',
            },
            81: {
                'target_group_id': 40,
                'campaign_type': 'Test/Control',
                'duration': 10,
                'lead_time': 0,
                'notes': '',
                'is_multi_channel': True,
                'is_recurrence': True,
                'status': 'Failed',
                'error': 'ESP unavailable',
            }
        })

    @responses.activate
    def test_get_executed_campaign_details_with_empty_date(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetExecutedCampaignDetails',
            callback=get_executed_campaign_details_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        self.assertRaises(Exception, client.actions.get_executed_campaign_details, None)

    @responses.activate
    def test_get_executed_campaign_details_with_wrong_date(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetExecutedCampaignDetails',
            callback=get_executed_campaign_details_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.actions.get_executed_campaign_details('3015-06-19')
        self.assertFalse(data)

    @responses.activate
    def test_get_campaign_details(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetCampaignDetails',
            callback=get_campaign_details_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.actions.get_campaign_details(21)
        self.assertEqual(data, {'campaign_id': 21, 'target_group_id': 15, 'campaign_type': 'Test/Control',
                                'duration': 7, 'lead_time': 3, 'notes': '', 'is_multi_channel': False,
                                'is_recurrence': False, 'status': 'Successful', 'error': ''})

    @responses.activate
    def test_get_campaign_details_with_empty_campaign_id(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetCampaignDetails',
            callback=get_campaign_details_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        self.assertRaises(Exception, client.actions.get_campaign_details, None)

    @responses.activate
    def test_get_campaign_details_with_wrong_campaign_id(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetCampaignDetails',
            callback=get_campaign_details_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.actions.get_campaign_details(12)
        self.assertFalse(data)

    @responses.activate
    def test_get_execution_channels(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetExecutionChannels',
            callback=get_execution_channels_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.actions.get_execution_channels()
        self.assertEqual(data, {
            1: 'Silverpop',
            2: 'ExactTarget',
            3: 'Lobby Banner',
            4: 'Call Center',
        })

    @responses.activate
    def test_get_executed_campaign_channel_details(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetExecutedCampaignChannelDetails',
            callback=get_executed_campaign_channel_details_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.actions.get_executed_campaign_channel_details(9214, 35)
        self.assertEqual(data, [{'campaign_id': 9214, 'channel_id': 35, 'list_id': 12,
                                'send_id': 'YTGF3C', 'template_id': 520, 'scheduled_time': '2015-12-30 08:00:00'},
                                {'campaign_id': 9214, 'channel_id': 35, 'send_id': '', 'template_id': 27,
                                 'scheduled_time': '2016-08-23 16:00:00'},
                                {'campaign_id': 9214, 'channel_id': 35, 'send_id': '', 'template_id': 1321,
                                 'scheduled_time': '2016-08-23 18:00:00'}])

    @responses.activate
    def test_get_executed_campaign_channel_details_empty_channel_id(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetExecutedCampaignChannelDetails',
            callback=get_executed_campaign_channel_details_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        self.assertRaises(Exception, client.actions.get_executed_campaign_channel_details, 9214, None)

    @responses.activate
    def test_get_executed_campaign_channel_details_wrong_channel_id(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetExecutedCampaignChannelDetails',
            callback=get_executed_campaign_channel_details_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.actions.get_executed_campaign_channel_details(9214, 34)
        self.assertFalse(data)

    @responses.activate
    def test_get_executed_campaigns_by_channel(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetExecutedCampaignsByChannel',
            callback=get_executed_campaigns_by_channel_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.actions.get_executed_campaigns_by_channel(35, '2016-02-20')
        self.assertEqual(data, [12, 16, 17, 19])

    @responses.activate
    def test_get_executed_campaigns_by_channel_empty_date(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetExecutedCampaignsByChannel',
            callback=get_executed_campaigns_by_channel_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        self.assertRaises(Exception, client.actions.get_executed_campaigns_by_channel, 35, None)

    @responses.activate
    def test_get_executed_campaigns_by_channel_wrong_date(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/actions/GetExecutedCampaignsByChannel',
            callback=get_executed_campaigns_by_channel_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.actions.get_executed_campaigns_by_channel(35, '3016-02-20')
        self.assertFalse(data)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


"""Constants"""
TOKEN = 'bf186b0a-ff34-44b2-9f62-496aa0d5e2c7'
HEADERS = {
    'text': {'Content-Type': 'text/plain'},
    'json': {'Content-Type': 'application/json'}
}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


class Actions(object):
    client = None

    def __init__(self, client):
        self.client = client

    def get_action_name(self, action_id):
        """Returns the action name associated with a particular action ID"""
        if not action_id:
            raise Exception('No ActionID provided')

        data = {
            'ActionID': action_id
        }

        response = self.client.get(self.client.get_url(), data)
        return response.json()['ActionName'] if response else False

    def get_action_id(self, action_name):
        """Returns the action ID associated with a particular action name."""
        if not action_name:
            raise Exception('No ActionName provided')

        data = {
            'ActionName': action_name
        }

        response = self.client.get(self.client.get_url(), data)
        return response.json()['ActionID'] if response else False

    def get_all_actions(self):
        """Returns all defined action IDs and corresponding action names."""
        response = self.client.get(self.client.get_url())

        results = {}
        for item in response.json():
            results[item['ActionID']] = item['ActionName']

        return results

    def get_actions_by_target_group(self, target_group_id, date):
        """Returns all the recipient group IDs and action IDs associated with a particular target group on
        a particular date."""
        if not target_group_id or not date:
            raise Exception('No TargetGroupID or Date provided')

        data = {
            'TargetGroupID': target_group_id,
            'Date': date
        }

        response = self.client.get(self.client.get_url(), data)
        if not response:
            return False

        results = {}
        for item in response.json():
            results[item['RecipientGroupID']] = item['ActionID']

        return results

    def get_promo_codes(self, date):
        """Returns all target group IDs, action IDs and promo codes for a particular date."""
        if not date:
            raise Exception('No Date provided')

        data = {
            'Date': date
        }

        response = self.client.get(self.client.get_url(), data)
        if not response:
            return False

        results = [{
                'target_group_id': item['TargetGroupID'],
                'recipient_group_id': item['RecipientGroupID'],
                'action_id': item['ActionID'],
                'promo_code': item['PromoCode']
            } for item in response.json()]

        return results

    def get_promo_codes_by_campaign(self, campaign_id):
        """Returns the recipient group IDs, action IDs and promo codes for a particular campaign ID."""
        if not campaign_id:
            raise Exception('No CampaignID provided')

        data = {
            'CampaignID': campaign_id
        }

        response = self.client.get(self.client.get_url(), data)
        if not response:
            return False

        results = [{
                'campaign_id': campaign_id,
                'recipient_group_id': item['RecipientGroupID'],
                'action_id': item['ActionID'],
                'promo_code': item['PromoCode']
            } for item in response.json()]

        return results

    def get_promo_codes_by_target_group(self, target_group_id, date):
        """Returns all recipient group IDs, action IDs and promo codes associated with a given target group on a
        particular date."""
        if not target_group_id or not date:
            raise Exception('No TargetGroupID or Date provided')

        data = {
            'TargetGroupID': target_group_id,
            'Date': date
        }

        response = self.client.get(self.client.get_url(), data)
        if not response:
            return False

        results = [{
                'target_group_id': target_group_id,
                'recipient_group_id': item['RecipientGroupID'],
                'action_id': item['ActionID'],
                'promo_code': item['PromoCode']
            } for item in response.json()]

        return results

    def get_action_details_by_target_group(self, target_group_id, date):
        """Returns the action IDs, action duration (in days), lead time (in days) and the execution channels associated
        with a given target group on a particular date."""
        if not target_group_id or not date:
            raise Exception('No TargetGroupID or Date provided')

        data = {
            'TargetGroupID': target_group_id,
            'Date': date
        }

        response = self.client.get(self.client.get_url(), data)
        if not response:
            return False

        results = [{
                'target_group_id': target_group_id,
                'recipient_group_id': item['RecipientGroupID'],
                'action_id': item['ActionID'],
                'duration': item['Duration'],
                'lead_time': item['LeadTime'],
                'channel_id': item['ChannelID']
            } for item in response.json()]

        return results

    def get_executed_campaign_details(self, date):
        """Returns details of every campaign executed on a particular date."""
        if not date:
            raise Exception('No Date provided')

        data = {
            'Date': date
        }

        response = self.client.get(self.client.get_url(), data)
        if not response:
            return False

        results = {}
        for item in response.json():
            results[item['CampaignID']] = {
                'target_group_id': item['TargetGroupID'],
                'campaign_type': item['CampaignType'],
                'duration': item['Duration'],
                'lead_time': item['LeadTime'],
                'notes': item['Notes'],
                'is_multi_channel': item['IsMultiChannel'] == 'true',
                'is_recurrence': item['IsRecurrence'] == 'true',
                'status': item['Status'],
                'error': item['Error'],
            }

        return results

    def get_campaign_details(self, campaign_id):
        """Returns details of a particular campaign."""
        if not campaign_id:
            raise Exception('No Date provided')

        data = {
            'CampaignID': campaign_id
        }

        response = self.client.get(self.client.get_url(), data)
        if not response:
            return False

        item = response.json()
        return {
            'campaign_id': campaign_id,
            'target_group_id': item['TargetGroupID'],
            'campaign_type': item['CampaignType'],
            'duration': item['Duration'],
            'lead_time': item['LeadTime'],
            'notes': item['Notes'],
            'is_multi_channel': item['IsMultiChannel'] == 'true',
            'is_recurrence': item['IsRecurrence'] == 'true',
            'status': item['Status'],
            'error': item['Error'],
        }

    def get_execution_channels(self):
        """Returns all available execution channels."""
        response = self.client.get(self.client.get_url())

        results = {}
        for item in response.json():
            results[item['ChannelID']] = item['ChannelName']
        return results

    def get_executed_campaign_channel_details(self, campaign_id, channel_id):
        """Returns the details of a particular channel used in a particular executed campaign."""
        if not campaign_id or not channel_id:
            raise Exception('No CampaignID or ChannelID provided')

        data = {
            'CampaignID': campaign_id,
            'ChannelID': channel_id
        }

        response = self.client.get(self.client.get_url(), data)
        if not response:
            return False

        results = []
        for item in response.json():
            result = {
                'campaign_id': campaign_id,
                'channel_id': channel_id,
                'send_id': item['SendID'],
                'template_id': item['TemplateID'],
                'scheduled_time': item['ScheduledTime']
            }
            if 'ListID' in item:
                result['list_id'] = item['ListID']
            results.append(result)
        return results

    def get_executed_campaigns_by_channel(self, channel_id, date):
        """Returns the list of campaigns executed for a particular channel on a particular date."""
        if not channel_id or not date:
            raise Exception('No ChannelID or Date provided')

        data = {
            'ChannelID': channel_id,
            'Date': date
        }

        response = self.client.get(self.client.get_url(), data)
        return [item['CampaignID'] for item in response.json()] if response else False



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from setuptools import setup, find_packages
import unittest

import optimove

setup(
    name='optimove',
    version=optimove.__version__,
    packages=find_packages(),
    author='Nicolas RAMY',
    author_email='nicolas.ramy@darkelda.com',
    license='MIT',
    description='This library allows you to quickly and easily use the Optimove Web API v3 via Python',
    long_description=open('README.rst').read(),
    include_package_data=True,
    install_requires=['requests', 'urllib3[secure]'],
    tests_require=['mock', 'responses'],
    test_suite='tests',
    url='https://github.com/nicolasramy/optimove',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7'
    ],
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from constants import AUTHORIZED_DELIMITERS, UNAUTHORIZED_DELIMITERS


class Customers(object):
    client = None

    def __init__(self, client):
        self.client = client

    def get_customers_by_action(self, recipient_group_id, action_id, date, attributes=None, delimiter=';',
                                top=None, skip=None):
        """Returns the list of customer IDs associated with a particular recipient group and action on
        a particular date, plus an optional customer attribute."""
        if not recipient_group_id or not action_id or not date:
            raise Exception('No RecipientGroupID, ActionID and Date provided')

        data = {
            'RecipientGroupID': recipient_group_id,
            'ActionID': action_id,
            'Date': date
        }

        if attributes and type(attributes) == list:
            data['CustomerAttributes'] = ';'.join(attributes)

            if delimiter:
                if delimiter in AUTHORIZED_DELIMITERS and delimiter not in UNAUTHORIZED_DELIMITERS:
                    data['CustomerAttributesDelimiter'] = delimiter
                else:
                    raise Exception('Invalid delimiter')

        if top and type(top) == int:
            data['$top'] = top

        if skip and type(skip) == int:
            data['$skip'] = skip

        response = self.client.get(self.client.get_url(), data)
        if not response:
            return False

        if attributes and type(attributes) == list:
            results = list()
            for item in response.json():
                result = {
                    'customer_id': item['CustomerID'],
                    'attributes': {
                        key: value for key, value in zip(attributes, item['CustomerAttributes'])
                    }
                }
                results.append(result)

        else:
            results = [item['CustomerID'] for item in response.json()]

        return results

    def get_customer_actions_by_target_group(self, target_group_id, date,
                                             include_control_group=False,
                                             attributes=None, delimiter=';',
                                             include_recipient_group_id=False, channel_id=None,
                                             top=None, skip=None):
        """Returns a list of customers and the details of the marketing actions they received, for a
        particular target group ID on a particular date."""
        if not target_group_id or not date:
            raise Exception('No TargetGroupID and Date provided')

        data = {
            'TargetGroupID': target_group_id,
            'Date': date
        }

        if channel_id:
            data['ChannelID'] = channel_id

        if include_recipient_group_id:
            data['IncludeRecipientGroupID'] = True

        if include_control_group:
            data['IncludeControlGroup'] = True

        if attributes and type(attributes) == list:
            data['CustomerAttributes'] = ';'.join(attributes)

            if delimiter:
                if delimiter in AUTHORIZED_DELIMITERS and delimiter not in UNAUTHORIZED_DELIMITERS:
                    data['CustomerAttributesDelimiter'] = delimiter
                else:
                    raise Exception('Invalid delimiter')

        if top and type(top) == int:
            data['$top'] = top

        if skip and type(skip) == int:
            data['$skip'] = skip

        response = self.client.get(self.client.get_url(), data)
        if not response:
            return False

        results = list()
        for item in response.json():
            result = {
                'customer_id': item['CustomerID'],
                'action_id': item['ActionID'],
                'channel_id': item['ChannelID']
            }
            if attributes and type(attributes) == list:
                result['attributes'] = {
                    key: value for key, value in zip(attributes, item['CustomerAttributes'])
                }

            results.append(result)

        return results

    def get_customer_one_time_actions_by_date(self, date, include_control_group=False, attributes=None, delimiter=';',
                                              top=None, skip=None):
        """Returns a list of customers and the details of the marketing actions they received as part of one-time
        campaigns executed on a particular date."""
        if not date:
            raise Exception('No Date provided')

        data = {
            'Date': date
        }

        if include_control_group:
            data['IncludeControlGroup'] = True

        if attributes and type(attributes) == list:
            data['CustomerAttributes'] = ';'.join(attributes)

            if delimiter:
                if delimiter in AUTHORIZED_DELIMITERS and delimiter not in UNAUTHORIZED_DELIMITERS:
                    data['CustomerAttributesDelimiter'] = delimiter
                else:
                    raise Exception('Invalid delimiter')

        if top and type(top) == int:
            data['$top'] = top

        if skip and type(skip) == int:
            data['$skip'] = skip

        response = self.client.get(self.client.get_url(), data)
        if not response:
            return False

        results = list()
        for item in response.json():
            result = {
                'customer_id': item['CustomerID'],
                'action_id': item['ActionID'],
                'channel_id': item['ChannelID']
            }
            if attributes and type(attributes) == list:
                result['attributes'] = {
                    key: value for key, value in zip(attributes, item['CustomerAttributes'])
                }
            results.append(result)

        return results

    def get_target_group_changers(self, start_date, end_date, attributes=None, delimiter=';', top=None, skip=None):
        """Returns the before and after target group IDs for customers whose target group changed during a particular
        date range."""
        if not start_date or not end_date:
            raise Exception('No StartDate and EndDate provided')

        data = {
            'StartDate': start_date,
            'EndDate': end_date
        }

        if attributes and type(attributes) == list:
            data['CustomerAttributes'] = ';'.join(attributes)

            if delimiter:
                if delimiter in AUTHORIZED_DELIMITERS and delimiter not in UNAUTHORIZED_DELIMITERS:
                    data['CustomerAttributesDelimiter'] = delimiter
                else:
                    raise Exception('Invalid delimiter')

        if top and type(top) == int:
            data['$top'] = top

        if skip and type(skip) == int:
            data['$skip'] = skip

        response = self.client.get(self.client.get_url(), data)
        if not response:
            return False

        results = list()
        for item in response.json():
            result = {
                'customer_id': item['CustomerID'],
                'initial_target_group_id': item['InitialTargetGroupID'],
                'final_target_group_id': item['FinalTargetGroupID']
            }
            if attributes and type(attributes) == list:
                result['attributes'] = {
                    key: value for key, value in zip(attributes, item['CustomerAttributes'])
                }
            results.append(result)

        return results

    def get_customer_attribute_changers(self, start_date, end_date, changed_customer_attribute,
                                        attributes=None, delimiter=';', top=None, skip=None):
        """Returns an array of customer IDs, and their before and after attribute values, for customers whose selected
        attribute changed during a particular date range."""
        if not start_date or not end_date or not changed_customer_attribute:
            raise Exception('No StartDate, EndDate and ChangedCustomerAttribute provided')

        data = {
            'StartDate': start_date,
            'EndDate': end_date,
            'ChangedCustomerAttributes': changed_customer_attribute
        }

        if attributes and type(attributes) == list:
            data['CustomerAttributes'] = ';'.join(attributes)

            if delimiter:
                if delimiter in AUTHORIZED_DELIMITERS and delimiter not in UNAUTHORIZED_DELIMITERS:
                    data['CustomerAttributesDelimiter'] = delimiter
                else:
                    raise Exception('Invalid delimiter')

        if top and type(top) == int:
            data['$top'] = top

        if skip and type(skip) == int:
            data['$skip'] = skip

        response = self.client.get(self.client.get_url(), data)
        if not response:
            return False

        results = list()
        for item in response.json():
            result = {
                'customer_id': item['CustomerID'],
                'initial_customer_attribute': None if item['InitialCustomerAttributes'] == 'NULL'
                else item['InitialCustomerAttributes'],
                'final_customer_attribute': None if item['FinalCustomerAttributes'] == 'NULL'
                else item['FinalCustomerAttributes']
            }
            if attributes and type(attributes) == list:
                result['attributes'] = {
                    key: value for key, value in zip(attributes, item['CustomerAttributes'])
                }
            results.append(result)

        return results

    def get_customer_future_values(self, life_cycle_stage_id=None, attribute=None, attribute_value=None,
                                   top=None, skip=None):
        """Returns customer IDs and their current future values."""
        if not life_cycle_stage_id and not attribute and not attribute_value:
            raise Exception('No LifecycleStageID or CustomerAttribute and CustomerAttributeValue provided')

        if life_cycle_stage_id and not attribute and not attribute_value:
            data = {'LifecycleStageID': life_cycle_stage_id}

        elif not life_cycle_stage_id and attribute and attribute_value:
            data = {
                'CustomerAttributes': attribute,
                'CustomerAttributeValue': attribute_value
            }

        else:
            raise Exception('Wrong combination for LifecycleStageID, CustomerAttribute and CustomerAttributeValue')

        if top and type(top) == int:
            data['$top'] = top

        if skip and type(skip) == int:
            data['$skip'] = skip

        response = self.client.get(self.client.get_url(), data)
        if not response:
            return False

        results = {}
        for item in response.json():
            results[item['CustomerID']] = item['FutureValue']

        return results

    def get_customer_last_action_executed(self, customer_id):
        """Returns details of the last action executed for a particular customer ID."""
        if not customer_id:
            raise Exception('No CustomerID provided')

        data = {
            'CustomerID': customer_id
        }

        response = self.client.get(self.client.get_url(), data)
        if not response:
            return False

        item = response.json()
        return {
            'customer_id': customer_id,
            'action_id': item['ActionID'],
            'date': item['Date'],
            'duration': item['Duration'],
            'target_group_id': item['TargetGroupID']
        }

    def get_customer_action_details_by_date(self, date, top=None, skip=None):
        """Returns customer IDs and details of the campaigns sent to them on a particular date."""
        if not date:
            raise Exception('No Date provided')

        data = {
            'Date': date
        }

        if top and type(top) == int:
            data['$top'] = top

        if skip and type(skip) == int:
            data['$skip'] = skip

        response = self.client.get(self.client.get_url(), data)
        if not response:
            return False

        results = list()
        for item in response.json():
            result = {
                'customer_id': item['CustomerID'],
                'recipient_group_id': item['RecipientGroupID'],
                'action_id': item['ActionID'],
                'channel_id': item['ChannelID']
            }
            results.append(result)

        return results

    def get_customers_action_ended_by_date(self, date, top=None, skip=None):
        """Returns customer IDs and details of the campaigns they received, for action durations which ended on a
        particular date."""
        if not date:
            raise Exception('No Date provided')

        data = {
            'Date': date
        }

        if top and type(top) == int:
            data['$top'] = top

        if skip and type(skip) == int:
            data['$skip'] = skip

        response = self.client.get(self.client.get_url(), data)
        if not response:
            return False

        results = list()
        for item in response.json():
            result = {
                'customer_id': item['CustomerID'],
                'target_group_id': item['TargetGroupID'],
                'action_id': item['ActionID'],
                'date': item['Date'],
                'duration': item['Duration'],
                'channel_id': item['ChannelID']
            }
            results.append(result)

        return results

    def get_customer_send_details_by_campaign(self, campaign_id, include_templates_ids=False, top=None, skip=None):
        """Returns an array of all customer IDs, channel IDs, send times and channel send IDs for
        a particular campaign ID."""
        if not campaign_id:
            raise Exception('No CampaignID provided')

        data = {
            'CampaignID': campaign_id
        }

        if include_templates_ids:
            data['IncludeTemplateIDs'] = True

        if top and type(top) == int:
            data['$top'] = top

        if skip and type(skip) == int:
            data['$skip'] = skip

        response = self.client.get(self.client.get_url(), data)
        if not response:
            return False

        results = list()
        for item in response.json():
            result = {
                'customer_id': item['CustomerID'],
                'channel_id': item['ChannelID'],
                'scheduled_time': item['ScheduledTime'],
                'send_id': item['SendID']
            }
            if include_templates_ids:
                result['template_id'] = item['TemplateID']
            results.append(result)

        return results

    def get_customer_send_details_by_channel(self, channel_id, campaign_id, attributes=None, delimiter=';',
                                             top=None, skip=None):
        """Returns an array of all customer IDs, template IDs, send times and customer attributes for a particular
        combination of channel ID and campaign ID."""
        if not channel_id or not campaign_id:
            raise Exception('No ChannelID and CampaignID provided')

        data = {
            'ChannelID': channel_id,
            'CampaignID': campaign_id
        }

        if attributes and type(attributes) == list:
            data['CustomerAttributes'] = ';'.join(attributes)

            if delimiter:
                if delimiter in AUTHORIZED_DELIMITERS and delimiter not in UNAUTHORIZED_DELIMITERS:
                    data['CustomerAttributesDelimiter'] = delimiter
                else:
                    raise Exception('Invalid delimiter')

        if top and type(top) == int:
            data['$top'] = top

        if skip and type(skip) == int:
            data['$skip'] = skip

        response = self.client.get(self.client.get_url(), data)
        if not response:
            return False

        results = list()
        for item in response.json():
            result = {
                'customer_id': item['CustomerID'],
                'template_id': item['TemplateID'],
                'scheduled_time': item['ScheduledTime']
            }
            if attributes and type(attributes) == list:
                result['attributes'] = {
                    key: value for key, value in zip(attributes, item['CustomerAttributes'])
                }
            results.append(result)

        return results

    def get_currently_targeted_customers(self, top=None, skip=None):
        """Returns an array of all customer IDs currently included in one or more campaigns."""

        data = {}
        if top and type(top) == int:
            data['$top'] = top

        if skip and type(skip) == int:
            data['$skip'] = skip

        response = self.client.get(self.client.get_url()) if not data else self.client.get(self.client.get_url(), data)

        results = list()
        for item in response.json():
            result = {
                'customer_id': item['CustomerID'],
                'campaign_id': item['CampaignID'],
                'action_id': item['ActionID'],
                'start_date': item['StartDate'],
                'end_date': item['EndDate']
            }
            results.append(result)

        return results

    def get_canceled_campaign_customers(self, campaign_id, top=None, skip=None):
        """Returns an array of all customer IDs that had been included in a campaign that was canceled, along with their
        associated action IDs and promo codes."""
        if not campaign_id:
            raise Exception('No CampaignID provided')

        data = {'CampaignID': campaign_id}

        if top and type(top) == int:
            data['$top'] = top

        if skip and type(skip) == int:
            data['$skip'] = skip

        response = self.client.get(self.client.get_url(), data)
        if not response:
            return False

        results = list()
        for item in response.json():
            result = {
                'customer_id': item['CustomerID'],
                'action_id': item['ActionID'],
                'promo_code': item['PromoCode']
            }
            results.append(result)

        return results



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import json
import unittest

from optimove.client import Client
from optimove.general import General
from optimove.constants import DEFAULT_URL
import responses

from constants import HEADERS, TOKEN
from helpers import login_callback, token_required


"""Callbacks"""


@token_required
def get_last_data_update_callback(request):
    resp_body = {'Date': '2015-08-13'}
    return 200, HEADERS['json'], json.dumps(resp_body)


@token_required
def register_event_listener_callback(request):
    payload = json.loads(request.body)
    if payload['EventTypeID'] == 1 and payload['ListenerURL'] == 'http%3A%2F%2Fwww.exampleurl.com%2Feventlistener6':
        resp_body = {'ResponseCode': 200}
        return 200, HEADERS['json'], json.dumps(resp_body)
    else:
        return 404, HEADERS['text'], 'Not Found'


@token_required
def unregister_event_listener_callback(request):
    payload = json.loads(request.body)
    if payload['EventTypeID'] == 1:
        resp_body = {'ResponseCode': 200}
        return 200, HEADERS['json'], json.dumps(resp_body)
    else:
        return 404, HEADERS['text'], 'Not Found'


"""Tests"""


class TestGeneral(unittest.TestCase):

    @responses.activate
    def test_login_success(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        self.assertEqual(client.token, TOKEN)

    @responses.activate
    def test_login_fail(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        client = Client()
        self.assertRaises(Exception, client.general.login, 'username', 'wrong_password')

    @responses.activate
    def test_login_no_credentials(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        client = Client()
        self.assertRaises(Exception, client.general.login, None, None)

    @responses.activate
    def test_last_update_date(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.GET,
            DEFAULT_URL + '/current/general/GetLastDataUpdate',
            callback=get_last_data_update_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.general.get_last_data_update()
        self.assertEqual(data, '2015-08-13')

    @responses.activate
    def test_register_event_listener(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/RegisterEventListener',
            callback=register_event_listener_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.general.register_event_listener(General.EVENT_TYPE_CAMPAIGN_PROCESSED,
                                                      'http://www.exampleurl.com/eventlistener6')
        self.assertTrue(data)

    @responses.activate
    def test_register_event_listener_with_empty_url(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/RegisterEventListener',
            callback=register_event_listener_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        self.assertRaises(Exception, client.general.register_event_listener,
                          General.EVENT_TYPE_CAMPAIGN_PROCESSED, None)

    @responses.activate
    def test_unregister_event_listener(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/UnregisterEventListener',
            callback=unregister_event_listener_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        data = client.general.unregister_event_listener(General.EVENT_TYPE_CAMPAIGN_PROCESSED)
        self.assertTrue(data)

    @responses.activate
    def test_unregister_event_listener_with_empty_event_type(self):
        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/login',
            callback=login_callback,
            content_type='application/json'
        )

        responses.add_callback(
            responses.POST,
            DEFAULT_URL + '/current/general/UnregisterEventListener',
            callback=unregister_event_listener_callback,
            content_type='application/json'
        )

        client = Client('username', 'password')
        self.assertRaises(Exception, client.general.unregister_event_listener, None)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import json

from constants import TOKEN, HEADERS


def login_callback(request):
    payload = json.loads(request.body)
    if payload['Username'] == 'username' and payload['Password'] == 'password':
        resp_body = TOKEN
        return 200, HEADERS['json'], json.dumps(resp_body)

    else:
        return 401, HEADERS['text'], 'Wrong user / password combination'


def token_required(funcname):
    def token_required_wrapper(*args, **kwargs):
        if 'Authorization-Token' not in args[0].headers:
            return 401, HEADERS['text'], 'Missing Authorization-Token'

        if args[0].headers['Authorization-Token'] != TOKEN:
            return 403, HEADERS['text'], 'Unauthorized User'

        return funcname(*args, **kwargs)

    return token_required_wrapper



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


from datetime import datetime
from urllib import quote_plus


class General(object):
    client = None
    username = None
    password = None

    EVENT_TYPE_CAMPAIGN_PROCESSED = 1
    EVENT_TYPE_TODAY_CAMPAIGNS_HAS_BEEN_PROCESSED = 2
    EVENT_TYPE_DAILY_DATA_UPDATE_COMPLETED = 3
    EVENT_TYPE_CAMPAIGN_CANCELED = 4

    def __init__(self, client):
        self.client = client

    def login(self, username, password):
        """Returns the authentication token required for all other functions during a particular session"""
        if not username or not password:
            raise Exception('No credentials provided')

        self.username = username
        self.password = password

        data = {
            'Username': self.username,
            'Password': self.password
        }
        url = self.client.get_url()
        response = self.client.post(url, data, check_token=False)
        self.client.token = response.json() if response else False
        self.client.expire = datetime.utcnow() if self.client.token else None
        return self.client.token

    def get_last_data_update(self):
        """Returns the date of the most recently available customer data"""
        response = self.client.get(self.client.get_url())
        return response.json()['Date'] if response else False

    def register_event_listener(self, event_type_id, listener_url):
        """Specifies the URL of a listener to which Optimove will report events of the specified type (e.g., “campaign
        scheduled”)."""
        if not event_type_id or not listener_url:
            raise Exception('No EventTypeID and ListenerURL provided')

        data = {
            'EventTypeID': event_type_id,
            'ListenerURL': quote_plus(listener_url)
        }

        response = self.client.post(self.client.get_url(), data)
        return bool(response)

    def unregister_event_listener(self, event_type_id):
        """Instructs Optimove to stop reporting events of the specified event type
        to a previously-registered listener."""
        if not event_type_id:
            raise Exception('No EventTypeID provided')

        data = {
            'EventTypeID': event_type_id
        }

        response = self.client.post(self.client.get_url(), data)
        return bool(response)
